package com.livelike.flickersearchlibrary.api.utils

import android.util.Log
import com.livelike.flickersearchlibrary.api.model.response.ApiResponse
import com.livelike.flickersearchlibrary.api.model.response.FlickrResponse
import retrofit2.Response

/**
 * Executes given Flickr API call and handles errors or exceptions.
 *
 * @return [ApiResponse] with state and data depending on executed API call.
 */
internal suspend fun executeApiCall(apiCall: suspend () -> Response<FlickrResponse>): ApiResponse {
    return runCatching {
        val response = apiCall()
        if (response.isSuccessful) {
            ApiResponse.Success(response.body()?.photosPage?.photos ?: emptyList())
        } else {
            throw Exception(
                "Flickr API responded with error: ${
                    response.errorBody().toString()
                }"
            )
        }
    }.onFailure { exception: Throwable ->
        Log.d(
            "FlickrApiManager",
            "Exception occurred when executing API call: ${exception.message}"
        )
    }.getOrElse { exception: Throwable ->
        ApiResponse.Error(exception.message.orEmpty())
    }
}