package com.livelike.flickersearchlibrary.service

import com.livelike.flickersearchlibrary.ui.model.PhotoItem

/**
 * Interface for the service providing Flickr features.
 */
internal interface FlickrService {

    /**
     * Searches for photos and retrieves [List] of [PhotoItem]s. [numOfPhotos] will be returned on given [page].
     * If no search query is provided, empty list will be returned.
     *
     * @param searchQuery [String] text query for searching photos.
     * @param numOfPhotos [Int] number of photos to return.
     * @param page [Int] the page where the photos should be contained.
     * @return [List] of [PhotoItem]s matching given [searchQuery] or empty list if no query is provided.
     */
    suspend fun search(searchQuery: String?, numOfPhotos: Int, page: Int): List<PhotoItem>

    /**
     * Retrieves [numOfPhotos] recently published photos on Flickr on the given [page].
     *
     * @param numOfPhotos [Int] number of photos to return.
     * @param page [Int] the page where the photos should be contained.
     * @return [List] of recently published photos on Flickr.
     */
    suspend fun getRecentPhotos(numOfPhotos: Int, page: Int): List<PhotoItem>
}