package com.livelike.flickersearchlibrary.service.logic

import com.livelike.flickersearchlibrary.api.FlickrApi
import com.livelike.flickersearchlibrary.api.model.response.ApiResponse
import com.livelike.flickersearchlibrary.api.utils.executeApiCall
import com.livelike.flickersearchlibrary.service.FlickrService
import com.livelike.flickersearchlibrary.ui.model.PhotoItem

/**
 * Implementation of [FlickrService].
 */
internal class FlickrServiceLogic(private val api: FlickrApi) : FlickrService {

    private val cachedPhotos = mutableListOf<PhotoItem>()

    override suspend fun search(
        searchQuery: String?,
        numOfPhotos: Int,
        page: Int
    ): List<PhotoItem> {
        return if (!searchQuery.isNullOrBlank()) {
            val response = executeApiCall {
                api.search(searchQuery, numOfPhotos, page)
            }
            if (response is ApiResponse.Success) {
                val photoItems = response.body.map {
                    it.toPhotoItem()
                }
                cachedPhotos.clear()
                cachedPhotos.addAll(photoItems)
            }

            cachedPhotos
        } else {
            emptyList()
        }
    }

    override suspend fun getRecentPhotos(numOfPhotos: Int, page: Int): List<PhotoItem> {
        val response = executeApiCall {
            api.getRecent(numOfPhotos, page)
        }
        return when (response) {
            is ApiResponse.Success -> response.body.map { it.toPhotoItem() }
            is ApiResponse.Error -> emptyList()
        }
    }
}