package com.livelike.flickersearchlibrary.ui

import android.os.Bundle
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.SearchView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.livelike.flickersearchlibrary.databinding.ActivityFlickrPhotosBinding
import com.livelike.flickersearchlibrary.ui.adapter.FlickrPhotosAdapter

class FlickrPhotosActivity : AppCompatActivity() {

    private val viewModel: FlickrPhotosViewModel by viewModels()

    private val views: ActivityFlickrPhotosBinding by lazy {
        ActivityFlickrPhotosBinding.inflate(layoutInflater)
    }
    private val photosAdapter = FlickrPhotosAdapter()

    private companion object {
        private const val PHOTOS_DEFAULT_COLUMN_COUNT = 3
        private const val BOTTOM_DIRECTION = 1
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(views.root)
        setupViews()
        setupListeners()
        setupObservers()
        initPhotosList()
    }

    private fun setupViews() {
        views.gridView.apply {
            adapter = photosAdapter
            layoutManager = GridLayoutManager(
                this@FlickrPhotosActivity,
                PHOTOS_DEFAULT_COLUMN_COUNT
            )
        }
        views.searchView.requestFocus()
    }

    private fun setupListeners() {
        views.searchView.setOnSearchClickListener {
            search(views.searchView.query.toString())
        }

        views.searchView.setOnQueryTextListener(object : SearchView.OnQueryTextListener {
            override fun onQueryTextSubmit(query: String?): Boolean {
                search(query)
                return true
            }

            override fun onQueryTextChange(newText: String?): Boolean {
                return true
            }
        })

        views.gridView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                if (!recyclerView.canScrollVertically(BOTTOM_DIRECTION)) {
                    viewModel.loadMorePhotos()
                }
            }
        })
    }

    private fun setupObservers() {
        viewModel.photos.observe(this) {
            photosAdapter.appendItems(it)
        }
    }

    private fun initPhotosList() {
        lifecycleScope.launchWhenCreated {
            viewModel.getRecentPhotos()
        }
    }

    private fun search(searchQuery: String?) {
        photosAdapter.clearItems()
        viewModel.searchPhotos(searchQuery)
    }
}