package com.livelike.flickersearchlibrary.ui

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.livelike.flickersearchlibrary.Flickr
import com.livelike.flickersearchlibrary.ui.model.PhotoItem
import kotlinx.coroutines.launch

internal class FlickrPhotosViewModel : ViewModel() {

    private val _photos = MutableLiveData<List<PhotoItem>>()
    val photos: LiveData<List<PhotoItem>> = _photos

    private lateinit var visiblePhotosState: PhotosState

    fun getRecentPhotos() = viewModelScope.launch {
        _photos.postValue(Flickr.getRecentPhotos())
        visiblePhotosState = PhotosState(PhotoType.RECENT, 1, null)
    }

    fun searchPhotos(searchQuery: String?) = viewModelScope.launch {
        _photos.postValue(Flickr.search(searchQuery))
        visiblePhotosState = PhotosState(PhotoType.SEARCH, 1, searchQuery)
    }

    fun loadMorePhotos() = viewModelScope.launch {
        val nextPage = visiblePhotosState.currentPage + 1
        when (visiblePhotosState.type) {
            PhotoType.RECENT -> _photos.postValue(Flickr.getRecentPhotos(page = nextPage))
            PhotoType.SEARCH -> _photos.postValue(
                Flickr.search(
                    page = nextPage,
                    searchQuery = visiblePhotosState.searchQuery
                )
            )
        }
        visiblePhotosState = visiblePhotosState.copy(currentPage = nextPage)
    }

    private data class PhotosState(
        var type: PhotoType,
        var currentPage: Int,
        var searchQuery: String?
    )

    private enum class PhotoType {
        RECENT,
        SEARCH
    }
}