package com.livelike.flickersearchlibrary.ui.adapter

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.AsyncListDiffer
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.livelike.flickersearchlibrary.databinding.LayoutPhotoItemBinding
import com.livelike.flickersearchlibrary.ui.model.PhotoItem

class FlickrPhotosAdapter : RecyclerView.Adapter<FlickrPhotosAdapter.PhotoViewHolder>() {

    private val itemsListDiffer = AsyncListDiffer(this, PhotoItemsDiffCallback())

    fun appendItems(items: List<PhotoItem>) {
        val currentList = itemsListDiffer.currentList.toMutableList()
        currentList.addAll(items)
        itemsListDiffer.submitList(currentList)
    }

    fun clearItems() {
        itemsListDiffer.submitList(emptyList())
    }

    override fun onCreateViewHolder(
        parent: ViewGroup,
        viewType: Int
    ): FlickrPhotosAdapter.PhotoViewHolder {
        val view = LayoutPhotoItemBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
        return PhotoViewHolder(view)
    }

    override fun onBindViewHolder(holder: FlickrPhotosAdapter.PhotoViewHolder, position: Int) {
        val item = itemsListDiffer.currentList[position]
        holder.bind(item)
    }

    override fun getItemCount() = itemsListDiffer.currentList.size

    inner class PhotoViewHolder(private val view: LayoutPhotoItemBinding) :
        RecyclerView.ViewHolder(view.root) {

        fun bind(photoItem: PhotoItem) {
            Glide
                .with(view.root)
                .load(photoItem.imageUrl)
                .centerCrop()
                .into(view.imageView)
        }
    }
}